%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM. Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%%  Copyright IBM Corporation 2011 
%% U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

function sl_customization(cm)

  %% Register custom menu function.
  cm.addCustomMenuFcn('Simulink:ToolsMenu', @getMyMenuItems);
end

%% Define the custom menu function.
function schemaFcns = getMyMenuItems(callbackInfo) 
  schemaFcns = {@menuRDM,...
                }; 
end

%% Define the schema function for group menu item.
function schema = menuRDM(callbackInfo)
  schema = sl_container_schema;
  schema.label = 'Rhapsody Design Manager';
  schema.childrenFcns = {@menuImport, @menuNavigateToServer};
end

function schema = menuImport(callbackInfo)
  schema = sl_action_schema;
  schema.label = 'Export model...';
  schema.userdata = 'publish model to RDM server';	
  schema.callback = @pulbishToServer;
end 

function pulbishToServer(callbackInfo)
    
    loadJavaObjects();

    launcher = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.DMLauncher');
     if isjava(launcher) == 0 
        fprintf('ERROR: java object (launcher) allocation failed\n');
     else
        path = which (bdroot);
        importerDir = getImporterFolder();
        launcher.launchImporter(importerDir, path);
     end
end

function schema = menuNavigateToServer(callbackInfo)
  schema = sl_action_schema;
  schema.label = 'Navigate to Design Manager...';
  schema.userdata = 'navigate to server';	
  schema.callback = @navigateToServer;
end 

function navigateToServer(callbackInfo)
    
    loadJavaObjects();

    launcher = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.DMLauncher');
     if isjava(launcher) == 0 
        fprintf('ERROR: java object (launcher) allocation failed\n');
     else
        path = which (bdroot);
        importerDir = getImporterFolder();
        
        sys = load_system(path);
        name = get_param(sys, 'name');

        launcher.launchDM(importerDir, name);
     end
end

function loadJavaObjects()
    myPath = mfilename('fullpath');
    slashedPath = regexprep(myPath, '\\', '/');
    javalib = regexprep(slashedPath, 'mscripts/sl_customization', 'domains/simulink/bin');
    
    files = dir(javalib);
    nfiles = size(files);

    % add libraries to java path
    for x = 1:nfiles
        filename = files(x).name;
        if strcmp(filename,'.') == 0 && strcmp(filename,'..') == 0
            javaaddpath(strcat(javalib,'/', filename));
        end
    end

end

function importerDir = getImporterFolder()
    myPath = mfilename('fullpath');
    slashedPath = regexprep(myPath, '\\', '/');
    importerDir = regexprep(slashedPath, 'mscripts/sl_customization', '');
end

